
#
# This is an example of applying the high-dimensional knockoff filter based on two steps 
# proposed in Candes's paper 2016.


#  control the FDR for Ultra-high dimensional regression using Knockfilter
#  In the first step: screening step using the Lasso: biglasso
#  In the scend step: variable selection based on the spliting data and recycling data


# detect the OS & set working directory

setwd("~/Documents/GKnockoff")
source('~/Documents/GKnockoff/gknockoff_all.R')
source('~/Documents/GKnockoff/isee_all.R')
source('~/Documents/GKnockoff/HDKnockoff_all.R')

require(graphics)
library(igraph)
library(MASS) 
library(Matrix)
library(pracma)

library(foreach)
library(glmnet)
library(lars)
library(scalreg)
library(RJSONIO)
library(knockoff)


set.seed(30)
# simulation parameters
n = 300          # number of observations
p = 2000          # number of variables
k = 30           # number of variables with nonzero coefficients
q = 0.2          # FDR level
a = 0.5
iter = 100     # iterative time
Hfdp       = rep(0,iter)
Htdp       = rep(0,iter)
Hfdp_plus  = rep(0,iter)
Htdp_plus  = rep(0,iter)
Hfdp_Rec   = rep(0,iter)
Htdp_Rec   = rep(0,iter)
Hfdp_plus_Rec  = rep(0,iter)
Htdp_plus_Rec  = rep(0,iter)
AS         = 1.5  # AS is signal amplitude (for noise level = 1)
amplitude  = c(-AS, AS)  # signal amplitude
cv.id      = vector(mode = "numeric", length = iter)

for(ii in 1:iter){
####################################################
### generate the variable X ########################
obj      = ar1.mat(a, p)
Sig.half = obj$Sigma.half
Ome.true = obj$Omega
X        = make.data1(Sig.half, n, p)
###################################################
###  get the true beta ############################

nonzero = sample(p, k)
beta.true = rep(0, p)
beta.true[nonzero] = sample(amplitude, k, replace = T)
id.true   = which(beta.true!=0)
###################################################
### generate the noise ###########################
sig = 1
eps = sig*matrix(rnorm(n),n,1)
##################################################
## consider the following linear model
y = X%*%beta.true + eps
#################################################
#### splite the sample 
SSample = sample.Splite(X,y)
X0      = SSample$X0
y0      = SSample$y0
X1      = SSample$X1
y1      = SSample$y1
#################################################
### using the data to do screening and get a selected set
Obj.scr.set = sel.set(X0, y0, n)
scr.set     = Obj.scr.set$scr.set
## judge whether the selected model including the true model
cv.id[ii] <- as.numeric(all(id.true %in% scr.set))

################################################
## based on selected set to get the reduced model and select the 
## variable using knockoff and knockoff plus respectively
sel.v   = sel.variable(scr.set, X0, X1, beta.true)
X1.S    = sel.v$X1.sel
X0.S    = sel.v$X0.sel
beta.S  = sel.v$beta.sel
res      = knockoff.filter(X1.S, y1, fdr=q, threshold = "knockoff")
res.plus = knockoff.filter(X1.S, y1, fdr=q, threshold = "knockoff+")

res.S     = res$selected
res.Splus = res.plus$selected
################################################
## compute the FDR and Power using spliting data

Hfdp[ii] = Hfdr(res.S, beta.S)      ## Knockoff FDR
Htdp[ii] = Hpow(res.S, beta.S)      ## Knockoff Power

Hfdp_plus[ii] = Hfdr(res.Splus, beta.S)  ## Knockoff plus FDR
Htdp_plus[ii] = Hpow(res.Splus, beta.S)  ## Knockoff plus Power

print(c(ii, Hfdp[ii], Htdp[ii], Hfdp_plus[ii], Htdp_plus[ii]))
#################################################

## Next we use the recycling data to compute the FDR and Power
normc = function(X) t(t(X)/sqrt(colSums(X^2)))
X0.S = normc(X0.S)
X1.S = normc(X1.S)
X1_KO = knockoff.create(X1.S)
X_S   = rbind(X0.S,X1.S)
X_KO  = rbind(X0.S, X1_KO)
y_S   = c(y0,y1)
W = knockoff.stat.lasso_difference(X_S, X_KO, y_S)
## W = knockoff.stat.lasso_signed_max(X_S, X_KO, y_S)
t_Rec      = knockoff.threshold(W, fdr = q, "knockoff")
S_Rec      = which(W >= t_Rec)
t_Rec_plus = knockoff.threshold(W, fdr = q, "knockoff+")
S_Rec_plus = which(W >= t_Rec_plus)
################################################
## compute the FDR and Power using recycling data

Hfdp_Rec[ii] = Hfdr(S_Rec, beta.S)      ## Knockoff FDR
Htdp_Rec[ii] = Hpow(S_Rec, beta.S)      ## Knockoff Power

Hfdp_plus_Rec[ii] = Hfdr(S_Rec_plus, beta.S)  ## Knockoff plus FDR
Htdp_plus_Rec[ii] = Hpow(S_Rec_plus, beta.S)  ## Knockoff plus Power

print(c(ii, Hfdp_Rec[ii], Htdp_Rec[ii], Hfdp_plus_Rec[ii], Htdp_plus_Rec[ii]))
#################################################



}

cp = sum(cv.id)/iter
print(cp)
print(c(n, p, a, mean(Hfdp), mean(Htdp), mean(Hfdp_plus), mean(Htdp_plus)))
print(c(n, p, a, mean(Hfdp_Rec), mean(Htdp_Rec), mean(Hfdp_plus_Rec), mean(Htdp_plus_Rec)))








  



