#
# This is an example of applying the graphical knockoff filter.
#
# The master file "isee_all.R" includes all the R functions for implementing the innovated 
# scalable efficient estimation (ISEE) procedure introduced in Fan and Lv (2015) for 
# scalable estimation of both the innovated data matrix (n x p matrix X tilde = X Omega) 
# and the precision matrix (Omega = inverse covariance matrix) in ultra-large scales.
#
# The ISEE framework has a range of potential applications involving precision 
# (innovated data) matrix including:
#   Linear and nonlinear classifications;
#   Dimension reduction; 
#   Portfolio management;
#   Feature screening and simultaneous inference.
#
# Written by 
# Yingying Fan (fanyingy@marshall.usc.edu) and Jinchi Lv (jinchilv@marshall.usc.edu)
# USC, April 10, 2014
#
# Reference:
# Fan, Y. and Lv, J. (2015). Innovated scalable efficient estimation in ultra-large 
# Gaussian graphical models. The Annals of Statistics, to appear.
#
# List of R functions for ISEE:
# Part I: main functions
#   1) isee: main function calculating hat matrix for Omega
#   2) isee.X: function calculating hat matrix for X tilde
#   3) slasso: function implementing tuning-free sparse regression with scaled Lasso
# Part II: additional functions
#   1) beta.block: function calculating beta hat matrix for each small block
#   2) isee.cv: cv function selecting threshold tau
# Part III: functions summarizing results and generating data matrix
#   1) rate.func: function summarizing selection results
#   2) make.data: function generating data matrix
#   3) blk.mat: function generating block diagonal Omega
#   4) ar1.mat: function generating AR(1) Omega
#   5) band.mat: function generating band Omega
#


# detect the OS & set working directory
# detect the OS & set working directory

setwd("/home/yanglu/Downloads/RANK/")
source('/home/yanglu/Downloads/RANK/gknockoff_all.R')
source('/home/yanglu/Downloads/RANK/isee_all.R')

# load packages
require(graphics)
library(igraph)
library(MASS) 
library(Matrix)
library(pracma)

# simulation parameters
n = 400
p = 200
obj = band.mat(a=0.5, p, K = 1)
Sig.half = obj$Sigma.half
Ome.true = obj$Omega
X = make.data(Sig.half, n, p, seed = 1000)

beta.true = c(rep(0, p))
S.true = c(1:30)
beta.true[S.true] = c(1,-1,1,-1,1,-1,1,1,-1,1,1,-1,1,-1,1,-1,1,1,-1,1,1,-1,1,-1,1,-1,1,1,-1,1)
sig = 0.5
q = 0.2

eps = sig*matrix(rnorm(n),n,1)
y = X%*%beta.true + eps

S = gknockoff(X, y, q, regfactor = "log"); print(S)
fdp = fdr(S, beta.true); print(fdp)
tdp = pow(S, beta.true); print(tdp)

